# sb::Admin::List
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::List;
use strict;
use sb::Utils ();
use sb::Build ();
use sb::App::Admin ();
use base qw( sb::App::Admin );
use vars qw( $VERSION );
$VERSION = '0.09';
# 0.09 [2009/01/29] modified _listmain to show date correctly
# 0.08 [2007/11/28] modified to use sb::Utils->remove_duplicates
# 0.07 [2007/09/16] added list order change marker
# 0.06 [2007/09/09] ported from Serene Bach 2.* / some subs have been moved to EntryList
# 0.05 [2006/11/01] changed _change_entry_status to change status of comment/trackback via the list
# 0.04 [2006/11/01] added _display_entry_statnum to output entry status class correctly
# 0.03 [2005/07/26] changed change_order to allow setting the order to the top
# 0.02 [2005/06/02] fixed a buf for displaying label of list related setting title attribute.
# 0.01 [2005/06/01] set title attribute for a list / load detail data for updating attachment.
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub JSFILE_NAME     (){ 'script/listtable.js' }
sub PAGELINK        (){ 'pagelink.html' };
sub COOKIE_CONF     (){ 'conf' };
sub MAX_LISTNUM     (){ 100 };
sub MIN_LISTNUM     (){ 10 };
sub DEFAULT_NUM     (){ 20 };
sub CATEGORY_LENGTH (){ 25 };
sub DENIED_CHECK    (){ '-' };
sub LIST_COLUMN     (){ 7 };
sub DATE_FORMAT     (){ '%YearShort%.%Mon%.%Day% %Hour%:%Min%' };
sub DATE_LANG       (){ 'en' };
sub DEFAULT_COLUMN  (){ '-' };
sub ORDER_LEFT      (){ '<input type="submit" name="up%d" value="&#9650;" class="updown" />' };
sub ORDER_RIGHT     (){ '<input type="submit" name="dn%d" value="&#9660;" class="updown" />' };
sub ORDER_COLUMN    (){ '%s</td><td>%s' };
my %mListSettings = ('dispnum' => DEFAULT_NUM);
my @mListNumArray = (20,50,100);
# --------------------------------------
# // override base class
sub js_paths
{
	my $self = shift;
	return $self->static_path . JSFILE_NAME;
}
# --------------------------------------
# // public functions - list utilities
sub setup_list
{
	my $self = shift;
	my %setting = %mListSettings;
	my $cookie = $self->cgi->cookie('name'=>$self->conf->value('basic_admntag') . COOKIE_CONF);
	foreach my $key ( keys(%setting) )
	{
		$setting{$key} = $cookie->{$key} if ( $cookie->{$key} ne '' );
		$setting{$key} = $self->cgi->value($key) if ( $self->cgi->value($key) ne '' );
	}
	$setting{'dispnum'} = MAX_LISTNUM if ($setting{'dispnum'} > MAX_LISTNUM);
	$setting{'dispnum'} = MIN_LISTNUM if ($setting{'dispnum'} < MIN_LISTNUM);
	$self->cgi->set_cookie(
		'time'   => $self->time->data,
		'name'   => $self->conf->value('basic_admntag') . COOKIE_CONF,
		'expire' => $self->conf->value('basic_admn_expire'),
		'path'   => $self->script_path,
		'data'   => \%setting,
	);
	return \%setting;
}
sub listmain
{
	my $self = shift;
	my %param = (
		'template'    => undef,
		'block'       => 'sb_list',
		'objects'     => [],
		'tags'        => {},
		'date_format' => DATE_FORMAT,
		@_
	);
	return if ( !defined($param{'template'}) );
	my $temp = $param{'template'};
	my $num = 0;
	foreach my $obj ( @{$param{'objects'}} )
	{
		$temp->num($num);
		foreach my $tag ( keys( %{$param{'tags'}} ) )
		{
			my $elem = $param{'tags'}->{$tag};
			my $data = '';
			if ($elem eq 'date' or $elem eq 'gen' or $elem eq 'mod')
			{ # date related elements
				if ( $obj->$elem() and $obj->$elem() =~ /^\d+$/ )
				{
					my $date = sb::Time->new($obj->$elem());
					$data = $date->format(
						'form' => $param{'date_format'},
						'zone' => ($elem eq 'date' and $obj->{'tz'} ne '') 
											? $obj->tz 
											: $self->conf->value('conf_timezone'),
						'lang' => DATE_LANG,
					);
				}
			}
			elsif ( ref($elem) eq 'CODE' )
			{ # get data via own handler
				eval{ $data = &$elem($self,$obj); };
			}
			else
			{ # normal handling
				$data = $obj->$elem();
			}
			$data = '' if ($data eq '');
			$temp->tag( $tag => $data );
		}
		$temp->tag('sb_list_class' => ($num % 2) ? 'odd' : 'even');
		$num++;
	}
	$temp->block($param{'block'} => $num);
	return $num;
}
sub linked_text
{
	my $self = shift;
	my %param = (
		'text'   => undef,
		'base'   => undef,
		'user'   => undef,
		'target' => undef,
		@_
	);
	my $attr = ($param{'target'}) ? ' target="' . $param{'target'} . '"' : '';
	my $url  = ($param{'base'} =~ /^\?/)
		? $self->script_path . $param{'base'} 
		: $param{'base'};
	return ( !defined($param{'user'}) or $self->has_permission('id'=>$param{'user'}) )
		? '<a href="' . $url . '"' . $attr . '>' . $param{'text'} . '</a>' 
		: $param{'text'};
}
sub clip_text
{
	my $self = shift;
	my %param = @_;
	$param{'length'} = 20 if (!$param{'length'});
	$param{'text'} = sb::Text->new($param{'text'})->clip('length'=>$param{'length'});
	return $self->linked_text(%param);
}
sub list_status
{
	my $self = shift;
	my %param = (
		'stat'   => 0,
		'string' => 'close:open',
		@_
	);
	return (split(':',$param{'string'}))[$param{'stat'}];
}
sub display_pagelink
{
	my $self = shift;
	my %param = (
		'mode'    => undef,
		'column'  => 1,
		'all'     => 0,
		'printed' => 0,
		'num'     => DEFAULT_NUM,
		'start'   => 0,
		'end'     => undef,
		'params'  => [],
		@_
	);
	return if ( !defined($param{'mode'}) );
	my $temp = sb::Template->new($self->load_template('file'=>PAGELINK));
	my $end  = defined($param{'end'}) ? $param{'end'} : int( $param{'all'} / $param{'num'} );
	my $page = ($self->cgi->value('page') eq '') ? $param{'start'} : int($self->cgi->value('page'));
	$page = int($self->cgi->value('now_page')) + 1 if ( $self->cgi->value('next') ne '' );
	$page = int($self->cgi->value('now_page')) - 1 if ( $self->cgi->value('prev') ne '' );
	$end--  if ( !defined($param{'end'}) and $param{'all'} % $param{'num'} == 0 and $param{'all'} > 0 );
	$self->common_parts($temp);
	$temp->num(0);
	$temp->tag('sb_list_all'=>$param{'all'});
	$temp->tag('sb_list_printed'=>$param{'printed'});
	$temp->tag('sb_pagenow'=>$page);
	$temp->tag('sb_pageprev'=>'disabled="disabled"') if ($page == $param{'start'});
	$temp->tag('sb_pagenext'=>'disabled="disabled"') if ($page == $end);
	my $selector = '';
	for (my $i=$param{'start'};$i<=$end;$i++)
	{
		$selector .= '<option value="' . $i . '"';
		$selector .= ' selected="selected"' if ($i == $page);
		$selector .= '>' . ($i + 1) . '</option>' if ($param{'start'} == 0);
		$selector .= '>' . $i . '</option>' if ($param{'start'} == 1);
	}
	$temp->tag('sb_pageselect'=>$selector);
	$temp->tag('sb_currentmode'=>$param{'mode'});
	$temp->tag('sb_table_column'=>$param{'column'});
	my $params = '';
	foreach my $key ( @{$param{'params'}} )
	{
		$params .= '<input type="hidden" name="' . $key . '" value="';
		$params .= sb::Text->new($self->cgi->value($key))->entitized() . '" />' . "\n";
	}
	$temp->tag('sb_list_params'=>$params);
	return $temp->output;
}
sub order_marker
{
	my $self = shift;
	my %param = (
		'template' => undef,
		'objects'  => [],
		'matched'  => 0,
		'tag'      => 'sb_list_order',
		'page'     => 0,
		'number'   => DEFAULT_NUM,
		@_
	);
	return if ( !defined($param{'template'}) );
	my $temp = $param{'template'};
	my @objs = @{$param{'objects'}};
	my $end = int( $param{'matched'} / $param{'number'} );
	$end-- if ( $param{'matched'} % $param{'number'} == 0 and $param{'matched'} > 0);
	for my $i (0 .. $#objs)
	{
		my $lcol = sprintf(ORDER_LEFT ,$objs[$i]->id);
		my $rcol = sprintf(ORDER_RIGHT,$objs[$i]->id);
		$lcol = DEFAULT_COLUMN if ($i == 0 and $param{'page'} == 0);
		$rcol = DEFAULT_COLUMN if ($i == $#objs and $param{'page'} == $end );
		$temp->num($i);
		$temp->tag($param{'tag'} => sprintf(ORDER_COLUMN,$lcol,$rcol));
	}
	return 1;
}
sub get_current_page
{
	my $self = shift;
	my $page = int($self->cgi->value('page'));
	$page = int($self->cgi->value('now_page')) + 1 if ( $self->cgi->value('next') ne '' );
	$page = int($self->cgi->value('now_page')) - 1 if ( $self->cgi->value('prev') ne '' );
	return $page;
}
# --------------------------------------
# // public functions - updater / builder
sub update_entry_attachment
{
	my $self = shift;
	my @eids = sb::Utils->remove_duplicates(@_);
	my @entries = sb::Data->load('Entry','cond'=>{'id'=>\@eids});
	foreach my $entry ( @entries )
	{
		my @com = sb::Data->load('Message','cond'=>{'eid'=>$entry->id,'stat'=>1});
		my @tb  = sb::Data->load('Trackback','cond'=>{'eid'=>$entry->id,'stat'=>1});
		$entry->com($#com + 1);
		$entry->tb($#tb + 1);
	}
	if (@entries)
	{
		sb::Data->update(@entries);
		$self->{'cat'} = { sb::Data->load_as_hash('Category') } if (!defined($self->{'cat'}));
		my $builder = sb::Build->new(
			'time'      => $self->time->data,
			'user'      => $self->users,
			'cat'       => $self->{'cat'},
			'sortedcat' => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
			'blog'      => sb::Data->load('Weblog','id'=>0),
		);
		foreach my $entry (@entries)
		{
			$self->build_entries('entry'=>$entry,'builder'=>$builder);
		}
	}
}
sub build_entries
{
	my $self = shift;
	my %param = (
		'entry'   => undef,
		'builder' => undef,
		@_
	);
	return if (!$param{'entry'} or !$param{'builder'});
	my $entry   = $param{'entry'};
	my $builder = $param{'builder'};
	if ($self->conf->value('conf_entry_archive') eq 'Individual')
	{
		$builder->build_entry( $entry ) if ($entry->stat != 0);
	}
	elsif ($self->conf->value('conf_entry_archive') eq 'Monthly')
	{
		my $month = sb::Time->new($entry->date)->format(
			'form'=>'%Year%%Mon%',
			'zone'=>$entry->tz
		);
		$builder->build_monthly_archive( $month );
	}
	return 1;
}
sub build_list
{
	my $self = shift;
	my $type = shift;
	$self->{'cat'} = { sb::Data->load_as_hash('Category') } if (!defined($self->{'cat'}));
	my $builder = sb::Build->new(
		'time'      => $self->time->data,
		'user'      => $self->users,
		'cat'       => $self->{'cat'},
		'sortedcat' => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
		'blog'      => sb::Data->load('Weblog','id'=>0),
	);
	if ($self->conf->value('conf_entry_archive') eq 'Individual' and $type ne '')
	{
		$builder->build_javascript($type);
	}
	$builder->build_top_page;
	return;
}
sub change_order
{
	my $self = shift;
	my %param = (
		'data'      => [],
		'target'    => undef,
		'direction' => undef,
		@_
	);
	return if ( !defined($param{'target'}) );
	my @objs = @{$param{'data'}};
	my $check = undef;
	for (my $i=0;$i<@objs;$i++)
	{
		$objs[$i]->order($#objs - $i);
		if ($objs[$i]->id == $param{'target'}->id)
		{
			$check = $i;
			$objs[$i]->order($#objs + 1) if ($param{'direction'} == 0 and $i > 0);
		}
	}
	if ($param{'direction'} == -1 and $check < $#objs)
	{
		$objs[$check]->order($#objs - $check - 1);
		$objs[$check + 1]->order($#objs - $check);
	}
	if ($param{'direction'} ==  1 and $check > 0)
	{
		$objs[$check]->order($#objs - $check + 1);
		$objs[$check - 1]->order($#objs - $check);
	}
	return @objs;
}
# --------------------------------------
# // public functions - selectors
sub dispnum_selector
{
	my $self = shift;
	my %param = (
		'cms'  => undef,
		'now'  => DEFAULT_NUM,
		'list' => \@mListNumArray,
		'half' => undef,
		@_
	);
	return if ( !defined($param{'cms'}) );
	my $cms = $param{'cms'};
	my $selector = '';
	for (my $i=0;$i<@{$param{'list'}};$i++)
	{
		my $num = ($param{'half'}) ? $param{'list'}->[$i] / 2 : $param{'list'}->[$i];
		$selector .= '<option value="' . $param{'list'}->[$i] . '"';
		$selector .= ' selected="selected"' if ($param{'list'}->[$i] == $param{'now'});
		$selector .= '>' . $num . '</option>';
	}
	$cms->num(0);
	$cms->tag('sb_dispnum_option'=>$selector);
}
sub select_option
{
	my $self = shift;
	my %param = (
		'cms'      => undef,
		'tag'      => undef,
		'selected' => undef,
		@_
	);
	return if ( !$param{'cms'} or !$param{'tag'} or $param{'selected'} eq '' );
	my $cms = $param{'cms'};
	$cms->num(0);
	$cms->tag($param{'tag'} . $param{'selected'}=>'selected="selected"');
}
sub monthly_selector
{
	my $self = shift;
	my %param = (
		'cms'  => undef,
		'tag'  => undef,
		'data' => undef,
		@_,
	);
	return if ( !$param{'cms'} or !$param{'tag'} or !$param{'data'} );
	my $cms = $param{'cms'};
	my %check = ();
	my @array = sb::Data->load($param{'data'},'sort'=>'date','order'=>1,'limit'=>'tz');
	for (my $i=0;$i<@array;$i++)
	{
		my $month = sb::Time->new($array[$i]->date)->format(
			'form'=>'%Year%%Mon%',
			'zone'=>$array[$i]->tz
		);
		if ( !defined($check{$month}) )
		{
			$check{$month} = sb::Time->new($array[$i]->date)->format(
				'form'=>$self->conf->value('conf_archivelist'),
				'zone'=>$array[$i]->tz
			);
		}
	}
	my $selector = '';
	my @monthly = sort { $b <=> $a } keys(%check);
	foreach my $month ( @monthly )
	{
		$selector .= '<option value="' . $month . '"';
		$selector .= ' selected="selected"' if ($month eq $self->cgi->value('dispdate'));
		$selector .= '>' . $check{$month} . '</option>';
	}
	$cms->num(0);
	$cms->tag($param{'tag'}=>$selector);
}
sub category_selector
{ # [note] need to set $self->{'cat'} as category hash ref. to call this method.
	my $self = shift;
	my %param = (
		'cat'    => [],
		'now'    => undef,
		'branch' => undef,   
		'main'   => undef,
		'except' => undef,
		'val'    => '%d',
		@_
	);
	return if (!$self->{'cat'});
	my $selector = '';
	foreach my $cat ( @{$param{'cat'}} )
	{
		my $parent = ($cat->main == -1) ? '' : $cat->main;
		next if ($param{'except'} ne '' and $cat->id eq $param{'except'});
		next if ($param{'main'} eq '' and $parent ne '');
		next if (defined($param{'branch'}) and $parent ne $param{'branch'});
		$selector .= '<option value="' . sprintf($param{'val'},$cat->id) . '"';
		$selector .= ($cat->id eq $param{'now'}) ? ' selected="selected">' : '>';
		$selector .= $cat->fullname($self->{'cat'}) . '</option>' . "\n";
		if ($cat->sub ne '')
		{
			$selector .= $self->category_selector(
				'cat'    => $param{'cat'},
				'branch' => $cat->id,
				'now'    => $param{'now'},
				'main'   => $cat->name,
				'except' => $param{'except'},
				'val'    => $param{'val'},
			);
		}
	}
	return $selector;
}
sub imagedir_selector
{
	my $self = shift;
	my %param = (
		'cms'    => undef,
		'tag'    => undef,
		'thumb'  => undef,
		'select' => $self->conf->value('conf_dir_img'),
		'format' => '%s',
		'over'   => undef,
		@_,
	);
	return if ( !$param{'cms'} or !$param{'tag'} );
	my $cms = $param{'cms'};
	my @dirs = sort { $a cmp $b } sb::Utils->writable_dir($self->conf->value('conf_dir_img'));
	my $selector = '';
	foreach my $dir (@dirs)
	{
		$selector .= '<option value="' . sprintf($param{'format'},$dir) . '"';
		$selector .= ' selected="selected"' if ($param{'select'} eq $dir);
		$selector .= '>' . $dir . '</option>';
	}
	$cms->num(0);
	$cms->tag($param{'tag'}=>$selector);
	if ($param{'thumb'})
	{
		$cms->block($param{'thumb'}=>1) if ( eval('require Image::Magick') );
		$cms->tag('sb_thumbchecked'=>'checked="checked"') if ($self->conf->value('conf_thumbcheck'));
	}
	$cms->block('sb_upload_overwrite'=>1) if ($param{'over'} and $self->conf->value('conf_imagename'));
}
sub template_selector
{
	my $self = shift;
	my %param = (
		'cms' => undef,
		'tag' => undef,
		'now' => undef,
		@_
	);
	return if ( !$param{'cms'} or !$param{'tag'} );
	my $cms = $param{'cms'};
	my @temps = sb::Data->load('Template');
	my $selector = '';
	foreach my $temp ( @temps ) {
		$selector .= '<option value="' . $temp->id . '"';
		$selector .= ' selected="selected"' if ($temp->id eq $param{'now'});
		$selector .= '>' . $temp->name . '</option>';
	}
	$cms->num(0);
	$cms->tag($param{'tag'}=>$selector);
}
# --------------------------------------
# // private functions - list handler
sub display_entry_title
{
	my $self = shift;
	my $obj  = shift;
	return $self->linked_text(
		'text' => ($obj->subj eq '') ? sb->string('[no title]') : $obj->subj,
		'base' => '?__mode=edit&amp;eid=' . $obj->id,
		'user' => $obj->auth,
	);
}
sub display_entry_status
{
	my $self = shift;
	my $obj  = shift;
	return $self->list_status(
		'stat'   => $self->get_entry_status($obj),
		'string' => sb->string('draft:open'),
	);
}
sub display_message_author
{
	my $self = shift;
	my $obj  = shift;
	my $elem = (ref($obj) eq 'sb::Data::Trackback') ? 'name' : 'auth';
	my $base = (ref($obj) eq 'sb::Data::Trackback') ? 'trackback&amp;bid=' : 'comment&amp;mid=';
	my $entry = sb::Data->load('Entry','id'=>$obj->eid);
	return $self->linked_text(
		'text' => ($obj->$elem() eq '') ? sb->string('[no name]') : $obj->$elem(),
		'base' => '?__mode=' . $base . $obj->id,
		'user' => $entry->auth,
	);
}
sub display_message_status
{
	my $self = shift;
	my $obj  = shift;
	return $self->list_status(
		'stat'   => $obj->stat,
		'string' => sb->string('wait:open:close'),
	);
}
sub get_entry_status
{
	my $self = shift;
	my $obj  = shift;
	return ($obj->stat == 0) ? 0 : 1;
}
sub display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return $self->has_permission('id'=>$obj->auth) 
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" class="form-control" />'
		: DENIED_CHECK;
}
1;
__END__
